package ch.fhnw.dlcopy.gui.swing;

import java.util.List;
import java.util.ResourceBundle;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 *
 * A panel with a list of files to overwrite.
 *
 * @author Ronny Standtke <ronny.standtke@gmx.net>
 */
public class OverwriteConfigurationPanel
        extends JPanel implements ListSelectionListener {

    private static final Logger LOGGER
            = Logger.getLogger(OverwriteConfigurationPanel.class.getName());
    private static final ResourceBundle STRINGS
            = ResourceBundle.getBundle("ch/fhnw/dlcopy/Strings");

    private final OverwriteTableModel model;

    /**
     * Creates new form OverwriteConfigurationPanel
     */
    public OverwriteConfigurationPanel() {
        initComponents();
        model = new OverwriteTableModel(table);
        table.setModel(model);
        table.getSelectionModel().addListSelectionListener(this);
        FileChooserCellEditor editor = new FileChooserCellEditor(this,
                STRINGS.getString("Select_Source"),
                STRINGS.getString("Select"),
                JFileChooser.FILES_AND_DIRECTORIES);
        table.getColumnModel().getColumn(0).setCellEditor(editor);
    }

    @Override
    public void valueChanged(ListSelectionEvent e) {
        if (e.getSource() == table.getSelectionModel()) {
            updateEnabledButtons();
        }
    }

    /**
     * returns an XML representation of the table model
     *
     * @return an XML representation of the table model
     */
    public String getXML() {
        return model.getXML();
    }

    /**
     * sets the table model data
     *
     * @param data the table model data
     */
    public void setXML(String data) {
        model.setXML(data);
    }

    /**
     * returns the list of table entries
     *
     * @return the list of table entries
     */
    public List<OverwriteEntry> getEntries() {
        return model.getEntries();
    }

    @Override
    public void setEnabled(boolean enabled) {
        super.setEnabled(enabled);
        updateEnabledButtons();
        table.setEnabled(enabled);
    }

    private void updateEnabledButtons() {
        addButton.setEnabled(isEnabled());
        int[] selectedRows = table.getSelectedRows();
        boolean selected = selectedRows.length > 0;
        removeButton.setEnabled(isEnabled() && selected);
        if (selected) {
            moveUpButton.setEnabled(isEnabled() && selectedRows[0] != 0);
            moveDownButton.setEnabled(isEnabled()
                    && (selectedRows[selectedRows.length - 1]
                    != model.getRowCount() - 1));
        } else {
            moveUpButton.setEnabled(false);
            moveDownButton.setEnabled(false);
        }
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        addButton = new javax.swing.JButton();
        removeButton = new javax.swing.JButton();
        moveUpButton = new javax.swing.JButton();
        moveDownButton = new javax.swing.JButton();
        scrollPane = new javax.swing.JScrollPane();
        table = new javax.swing.JTable();

        setLayout(new java.awt.GridBagLayout());

        addButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/16x16/list-add.png"))); // NOI18N
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("ch/fhnw/dlcopy/Strings"); // NOI18N
        addButton.setToolTipText(bundle.getString("DLCopySwingGUI.upgradeOverwriteAddButton.toolTipText")); // NOI18N
        addButton.setMargin(new java.awt.Insets(2, 2, 2, 2));
        addButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(30, 5, 0, 0);
        add(addButton, gridBagConstraints);

        removeButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/16x16/list-remove.png"))); // NOI18N
        removeButton.setToolTipText(bundle.getString("DLCopySwingGUI.upgradeOverwriteRemoveButton.toolTipText")); // NOI18N
        removeButton.setEnabled(false);
        removeButton.setMargin(new java.awt.Insets(2, 2, 2, 2));
        removeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                removeButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 0);
        add(removeButton, gridBagConstraints);

        moveUpButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/16x16/arrow-up.png"))); // NOI18N
        moveUpButton.setToolTipText(bundle.getString("DLCopySwingGUI.upgradeMoveUpButton.toolTipText")); // NOI18N
        moveUpButton.setEnabled(false);
        moveUpButton.setMargin(new java.awt.Insets(2, 2, 2, 2));
        moveUpButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moveUpButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(10, 5, 0, 0);
        add(moveUpButton, gridBagConstraints);

        moveDownButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/16x16/arrow-down.png"))); // NOI18N
        moveDownButton.setToolTipText(bundle.getString("DLCopySwingGUI.upgradeMoveDownButton.toolTipText")); // NOI18N
        moveDownButton.setEnabled(false);
        moveDownButton.setMargin(new java.awt.Insets(2, 2, 2, 2));
        moveDownButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moveDownButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.PAGE_START;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 10, 0);
        add(moveDownButton, gridBagConstraints);

        scrollPane.setViewportView(table);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridheight = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        add(scrollPane, gridBagConstraints);
    }// </editor-fold>//GEN-END:initComponents

    private void moveUpButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_moveUpButtonActionPerformed
        int selectedRows[] = table.getSelectedRows();
        model.moveUp(selectedRows);
        table.clearSelection();
        for (int selectedRow : selectedRows) {
            int previousRow = selectedRow - 1;
            table.addRowSelectionInterval(previousRow, previousRow);
        }
    }//GEN-LAST:event_moveUpButtonActionPerformed

    private void moveDownButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_moveDownButtonActionPerformed
        int selectedRows[] = table.getSelectedRows();
        model.moveDown(selectedRows);
        table.clearSelection();
        for (int selectedRow : selectedRows) {
            int nextRow = selectedRow + 1;
            table.addRowSelectionInterval(nextRow, nextRow);
        }
    }//GEN-LAST:event_moveDownButtonActionPerformed

    private void addButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addButtonActionPerformed
        model.addRow(new String[]{"", ""});
    }//GEN-LAST:event_addButtonActionPerformed

    private void removeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeButtonActionPerformed
        model.removeRows(table.getSelectedRows());
    }//GEN-LAST:event_removeButtonActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addButton;
    private javax.swing.JButton moveDownButton;
    private javax.swing.JButton moveUpButton;
    private javax.swing.JButton removeButton;
    private javax.swing.JScrollPane scrollPane;
    private javax.swing.JTable table;
    // End of variables declaration//GEN-END:variables
}
